# frozen_string_literal: true

require "forwardable"

require "statscloud/statscloud_client"
require "statscloud/version"

# StatsCloud.io ruby client
module StatsCloud
  class << self
    include LoggerHelper
    extend Forwardable

    def_delegators :stats_cloud_instance, :start, :with_environment, :with_tags, :meter, :record_event, :record_events,
                   :record_events_array, :cluster_status, :stop, :environment, :client_tags

    def clone_with_config_file(file)
      new_statscloud_instance(environment, client_tags).with_config_file(file)
    end

    private

    def stats_cloud_instance
      @stats_cloud_instance ||= new_statscloud_instance
    end

    def new_statscloud_instance(environment = nil, client_tags = nil)
      StatsCloud::Client.new(environment, client_tags)
    end
  end
end
