# frozen_string_literal: true

require_relative "logger_helper"
require_relative "../errors/client_error"

module StatsCloud
  # This helper configures StatsCloud plugins.
  module PluginsHelper
    private

    def build_plugins(plugins)
      plugins&.each do |plugin|
        plugin = plugin.is_a?(Hash) ? plugin : simple_plugin_config(plugin)
        plugin[:class_name] = constantize_plugin_by_name(plugin_class_name_from_plugin(plugin))
      end
    end

    def start_plugins_job(plugins, mutex)
      plugins&.each do |plugin|
        start_plugin(plugin, mutex)
      end
    end

    def simple_plugin_config(plugin)
      {
        "name": plugin
      }
    end

    def start_plugin(plugin, mutex)
      plugin[:class_name].start(self, mutex, plugin["settings"])
    end

    def constantize_plugin_by_name(plugin_name)
      Object.const_get "StatsCloud::Plugin::#{plugin_name}"
    rescue NameError
      raise statscloud_error(no_plugin_error_message(plugin_name))
    end

    def no_plugin_error_message(plugin_name)
      "Can not find StatsCloud::Plugin::#{plugin_name}. Please require gem to load this plugin"
    end

    def plugin_class_name_from_plugin(plugin)
      plugin["name"].split("-").map(&:capitalize).join
    end

    def statscloud_error(message)
      StatsCloud::ClientError.new(message)
    end
  end
end
