# frozen_string_literal: true

require "statscloud.io-ruby-socket.io-client-simple"

require_relative "../logger_helper"

module StatsCloud
  # This helper works to create socket connection.
  module SocketIOHelper
    private

    include StatsCloud::LoggerHelper

    def open?
      !@client.nil? && @client.open?
    end

    def connect_client
      eventmachine.run do
        eventmachine.add_periodic_timer(5) do
          try_connect
          eventmachine.add_timer(3) { eventmachine.stop if connected? }
        end
      end
    end

    def try_connect
      @client = socketio_client.connect @url, path: "/ws", "auth-token" => @token
      listen_connect
      listen_disconnect
      listen_events
      listen_errors
    rescue StandardError => error
      log_error(error)
      close
    end

    def listen_connect
      this = self

      @client.on :connect do
        emit :tags, this.tags
        this.logger.info "StatsCloud client has connected to StatsCloud server"
      end
    end

    def listen_disconnect
      this = self

      @client.on :disconnect do
        this.log_error("StatsCloud client disconnected from the StatsCloud server")
      end
    end

    def listen_events
      this = self

      @client.on :events do |names_map|
        this.names_map = names_map
        this.event_name_size_in_bytes = (Math.log(names_map.keys.length) / Math.log(256)).ceil
        this.event_name_size_in_bytes = 1 if this.event_name_size_in_bytes < 1
      end
    end

    def listen_errors
      this = self

      @client.on :error do |error|
        this.log_error error
      end
    end

    def eventmachine
      EM
    end

    def socketio_client
      ::StatsCloudIO::SocketIO::Client::Simple
    end
  end
end
