# frozen_string_literal: true

require "eventmachine"

require_relative "../helpers/logger_helper"

module StatsCloud
  # Registers StatsCloud connection.
  class RegisterConnectionJob
    include LoggerHelper

    ERROR_CODES = [400, 401, 404, 500].freeze

    def initialize(cluster_client, token, app, tags)
      @cluster_client = cluster_client
      @token = token
      @app = app
      @tags = tags
      @job_running = false
    end

    def start
      Thread.new do
        event_machine.run do
          event_machine.add_periodic_timer(3600) { run_job }
        end
      end.join
    end

    private

    attr_reader :cluster_client, :token, :app, :tags

    def run_job
      return if @job_running

      start_running
      connection_response = cluster_client.register_statscloud_connection(token, app, tags)
      return stop_running if connection_response.code == 204

      check_for_errors(connection_response)
    end

    def start_running
      @job_running = true
    end

    def stop_running
      @job_running = false
    end

    def check_for_errors(response)
      log_register_connection_error(response.body) if ERROR_CODES.include?(response.code)
    end

    def log_register_connection_error(response_body)
      log_error response_body[:message]
    end

    def event_machine
      EventMachine
    end
  end
end
